﻿using ELTE.BoardGame.Model;
using ELTE.BoardGame.Persistence;
using ELTE.BoardGame.View;
using ELTE.BoardGame.ViewModel;
using Microsoft.Win32;
using System;
using System.Windows;

namespace ELTE.BoardGame
{
    /// <summary>
    /// Alkalmazás típusa.
    /// </summary>
    public partial class App : Application
    {
        private IPersistence persistence;
        private BoardGameModel model;
        private BoardViewModel viewModel;
        private BoardGameWindow window;
        private OpenFileDialog openFileDialog;
        private SaveFileDialog saveFileDialog;

        /// <summary>
        /// Alkalmazás példányosítása.
        /// </summary>
        public App()
        {
            Startup += new StartupEventHandler(App_Startup);
        }

        /// <summary>
        /// Alkalmazás indulásának eseménykezelője.
        /// </summary>
        private void App_Startup(object sender, StartupEventArgs e)
        {
            persistence = new TextFilePersistence();

            model = new ConnectFourModel(10, persistence);
            model.GameWon += new EventHandler<GameWonEventArgs>(Model_GameWon);
            model.GameOver += new EventHandler(Model_GameOver);
            model.NewGame();

            viewModel = new BoardViewModel(model);
            viewModel.LoadGame += new EventHandler(ViewModel_LoadGame); // kezeljük a nézetmodell eseményeit
            viewModel.SaveGame += new EventHandler(ViewModel_SaveGame);
            viewModel.GameExit += new EventHandler(ViewModel_GameExit);

            window = new BoardGameWindow();
            window.DataContext = viewModel;
            window.Show();
        }

        #region Model event handlers

        /// <summary>
        /// Játék megnyerésének eseménykezelése.
        /// </summary>
        private void Model_GameWon(object sender, GameWonEventArgs e)
        {
            switch (e.Player)
            {
                case PlayerData.PlayerO:
                    MessageBox.Show("A kör játékos győzött!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
                    break;
                case PlayerData.PlayerX:
                    MessageBox.Show("A kereszt játékos győzött!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
                    break;
            }
            model.NewGame();
        }

        /// <summary>
        /// Játék végének eseménykezelése.
        /// </summary>
        private void Model_GameOver(object sender, EventArgs e)
        {
            MessageBox.Show("Döntetlen játék!", "Játék vége!", MessageBoxButton.OK, MessageBoxImage.Asterisk);
            model.NewGame();
        }

        #endregion

        #region ViewModel event handlers

        /// <summary>
        /// Játék betöltésének eseménykezelője.
        /// </summary>
        private void ViewModel_LoadGame(object sender, System.EventArgs e)
        {
            if (openFileDialog == null)
            {
                openFileDialog = new OpenFileDialog();
                openFileDialog.Title = "Tic-Tac-Toe - Játék betöltése";
                openFileDialog.Filter = "Szövegfájlok|*.txt";
            }

            // nyithatunk új nézetet
            if (openFileDialog.ShowDialog() == true)
            {
                try
                {
                    model.LoadGame(openFileDialog.FileName); // játék betöltése
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a betöltés során.", "Tic-Tac-Toe", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
        /// <summary>
        /// Játék mentésének eseménykezelője.
        /// </summary>
        private void ViewModel_SaveGame(object sender, System.EventArgs e)
        {
            if (saveFileDialog == null)
            {
                saveFileDialog = new SaveFileDialog();
                saveFileDialog.Title = "Tic-Tac-Toe - Játék mentése";
                saveFileDialog.Filter = "Szövegfájlok|*.txt";
            }

            if (saveFileDialog.ShowDialog() == true)
            {
                try
                {
                    model.SaveGame(saveFileDialog.FileName); // játék mentése
                }
                catch (DataException)
                {
                    MessageBox.Show("Hiba keletkezett a mentés során.", "Tic-Tac-Toe", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
        /// <summary>
        /// Kilépés eseménykezelője.
        /// </summary>
        private void ViewModel_GameExit(object sender, System.EventArgs e)
        {
            Shutdown(); // a teljes alkalmazás bezárása
        }

        #endregion
    }
}
