﻿using System;
using System.Collections.ObjectModel;
using System.Linq;
using ELTE.BoardGame.Model;

namespace ELTE.BoardGame.ViewModel
{
    /// <summary>
    /// Játéktábla nézetmodell típusa.
    /// </summary>
    public class BoardViewModel : ViewModelBase
    {
        #region Private fields

        private BoardGameModel model;

        #endregion

        #region Events

        /// <summary>
        /// Játékból való kilépés eseménye.
        /// </summary>
        public event EventHandler GameExit;

        /// <summary>
        /// Játék betöltésének eseménye.
        /// </summary>
        public event EventHandler LoadGame;

        /// <summary>
        /// Játék mentésének eseménye.
        /// </summary>
        public event EventHandler SaveGame;

        #endregion

        #region Constructors

        /// <summary>
        /// Nézetmodell példányosítása
        /// </summary>
        public BoardViewModel(BoardGameModel model)
        {
            if (model == null)
                throw new ArgumentNullException(nameof(model));

            this.model = model;
            this.model.FieldChanged += Model_FieldChanged;

            // parancsok kezelése
            this.NewGameCommand = new DelegateCommand(param => NewGame());
            this.LoadGameCommand = new DelegateCommand(param => OnLoadGame());
            this.SaveGameCommand = new DelegateCommand(param => OnSaveGame());

            // tábla létrehozása, amelynek változását követjük
            this.BoardItems = new ObservableCollection<BoardItemViewModel>();
            this.CreateBoard();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Új játék kezdése parancs lekérdezése.
        /// </summary>
        public DelegateCommand NewGameCommand { get; private set; }

        /// <summary>
        /// Játék betöltése parancs lekérdezése.
        /// </summary>
        public DelegateCommand LoadGameCommand { get; private set; }

        /// <summary>
        /// Játék mentése parancs lekérdezése.
        /// </summary>
        public DelegateCommand SaveGameCommand { get; private set; }

        /// <summary>
        /// Kilépés parancs lekérdezése.
        /// </summary>
        public DelegateCommand ExitGameCommand { get; private set; }

        /// <summary>
        /// Tábla szélességének lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 BoardWidth { get { return this.model.BoardWidth; } }

        /// <summary>
        /// Tábla magasságának lekérdezése, vagy beállítása.
        /// </summary>
        public Int32 BoardHeight { get { return this.model.BoardHeight; } }

        /// <summary>
        /// Tábla elemeinek lekérdezése, vagy beállítása.
        /// </summary>
        public ObservableCollection<BoardItemViewModel> BoardItems { get; private set; }

        #endregion

        #region Private methods

        /// <summary>
        /// Új játék indítása.
        /// </summary>
        private void NewGame()
        {
            this.model.NewGame();
        }

        /// <summary>
        /// Játékos átalakítása a megjelentéshez.
        /// </summary>
        /// <param name="player">Játékos.</param>
        /// <returns>Játékos szöveges megfelelője.</returns>
        private String GetDisplayedState(PlayerData player)
        {
            switch (player)
            {
                case PlayerData.PlayerX:
                    return "X";
                case PlayerData.PlayerO:
                    return "O";
                default:
                    return String.Empty;
            }
        }
        
        /// <summary>
        /// Tábla létrehozása.
        /// </summary>
        private void CreateBoard()
        {
            BoardItems.Clear();

            for (Int32 rowIndex = 0; rowIndex < this.model.BoardHeight; rowIndex++)
            {
                for (Int32 columnIndex = 0; columnIndex < this.model.BoardWidth; columnIndex++)
                {
                    BoardItems.Add(new BoardItemViewModel(param =>
                    {
                        Int32 number = Convert.ToInt32(param);
                        Int32 row = number / this.model.BoardWidth; 
                        Int32 column = number % this.model.BoardWidth;

                        try
                        {
                            this.model.StepGame(column, row);
                        }
                        catch { }
                    }) {
                        Player = GetDisplayedState(this.model[rowIndex, columnIndex]),
                        Row = rowIndex,
                        Column = columnIndex,
                        Number = rowIndex * this.model.BoardWidth + columnIndex
                    });
                }
            }
        }

        /// <summary>
        /// Modell mezőváltozásának eseménykezelése.
        /// </summary>
        private void Model_FieldChanged(object sender, FieldChangedEventArgs e)
        {
            BoardItems.FirstOrDefault(item => item.Row == e.RowIndex && item.Column == e.ColumnIndex).Player = GetDisplayedState(this.model[e.ColumnIndex, e.RowIndex]);
            // lineáris keresés a megadott sorra, oszlopra, majd a játékos átírása, ez automatikusan frissíti a megjelenítést
        }

        #endregion

        #region Event methods

        /// <summary>
        /// Játék betöltése eseménykiváltása.
        /// </summary>
        private void OnLoadGame()
        {
            if (this.LoadGame != null)
                this.LoadGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játék mentése eseménykiváltása.
        /// </summary>
        private void OnSaveGame()
        {
            if (this.SaveGame != null)
                this.SaveGame(this, EventArgs.Empty);
        }

        /// <summary>
        /// Játékból való kilépés eseménykiváltása.
        /// </summary>
        private void OnGameExit()
        {
            if (this.GameExit != null)
                this.GameExit(this, EventArgs.Empty);
        }

        #endregion
    }
}
